@@ -2,11 +2,12 @@ from ckan.plugins.interfaces import Interface
 
 
 class ITransmute(Interface):
-    '''
+    """
     Add extra transmutators to be used in tsm_transmute action
-    '''
+    """
+
     def get_transmutators(self):
-        u'''Return the transmutator functions provided by this plugin.
+        """Return the transmutator functions provided by this plugin.
 
         Return a dictionary mapping transmutator names (strings) to
         transmutator functions. For example::
@@ -16,4 +17,4 @@ class ITransmute(Interface):
 
         These transmutator functions would then be available for
         tsm_schema.
-        '''
+        """
